import React, { useMemo, useContext } from 'react';
import { ReactReduxContext } from 'react-redux';
import { subspace, parentSpace } from 'redux-subspace';
import hoistNonReactStatics from 'hoist-non-react-statics';
import wrapDisplayName from 'recompose/wrapDisplayName';
import PropTypes from 'prop-types';

var isObject = function isObject(value) {
  var type = typeof value;
  return type == "object" && value != null && !Array.isArray(value);
};

/**
 * Copyright 2017, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */
var useSubspaceAdvanced = function useSubspaceAdvanced(mapState, namespace, subspaceDecorator, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$context = _ref.context,
      context = _ref$context === void 0 ? ReactReduxContext : _ref$context;

  var subspacer = useMemo(function () {
    return subspaceDecorator || subspace(mapState, namespace);
  }, [mapState, namespace]);

  var _useContext = useContext(context),
      store = _useContext.store;

  return useMemo(function () {
    return subspacer(store);
  }, [subspacer, store]);
};

var useSubspace = function useSubspace(mapState, namespace, options) {
  if (options === undefined && isObject(namespace)) {
    return useSubspace(mapState, undefined, namespace);
  }

  return useSubspaceAdvanced(mapState, namespace, undefined, options);
};

/**
 * Copyright 2017, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

var useParentSpace = function useParentSpace(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$context = _ref.context,
      context = _ref$context === void 0 ? ReactReduxContext : _ref$context;

  var _useContext = useContext(context),
      store = _useContext.store;

  return useMemo(function () {
    return parentSpace(store);
  }, [store]);
};

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var useReplacedContext = function useReplacedContext(context, store) {
  var contextValue = useContext(context);
  return useMemo(function () {
    return _objectSpread2({}, contextValue, {
      store: store
    });
  }, [contextValue, store]);
};

/**
 * Copyright 2017, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

var SubspaceProvider = function SubspaceProvider(_ref) {
  var mapState = _ref.mapState,
      namespace = _ref.namespace,
      subspaceDecorator = _ref.subspaceDecorator,
      _ref$context = _ref.context,
      context = _ref$context === void 0 ? ReactReduxContext : _ref$context,
      children = _ref.children;
  var _context$parent = context.parent,
      ParentContext = _context$parent === void 0 ? context.Consumer ? context : ReactReduxContext : _context$parent,
      _context$child = context.child,
      ChildContext = _context$child === void 0 ? context.Consumer ? context : ReactReduxContext : _context$child;
  var subspacedStore = useSubspaceAdvanced(mapState, namespace, subspaceDecorator, {
    context: ParentContext
  });
  var childContext = useReplacedContext(ChildContext, subspacedStore);
  return React.createElement(ChildContext.Provider, {
    value: childContext
  }, children);
};

SubspaceProvider.propTypes = {
  children: PropTypes.element.isRequired,
  mapState:
  /*#__PURE__*/
  PropTypes.oneOfType([PropTypes.func, PropTypes.string]),
  namespace: PropTypes.string,
  subspaceDecorator: PropTypes.func
};

var subspaced = function subspaced(mapState, namespace, subspaceOptions) {
  if (subspaceOptions === undefined && isObject(namespace)) {
    return subspaced(mapState, undefined, namespace);
  }

  var subspaceDecorator = subspace(mapState, namespace);
  return function (WrappedComponent) {
    var SubspacedComponent = function SubspacedComponent(props) {
      return React.createElement(SubspaceProvider, _extends({}, subspaceOptions, {
        subspaceDecorator: subspaceDecorator
      }), React.createElement(WrappedComponent, props));
    };

    hoistNonReactStatics(SubspacedComponent, WrappedComponent);
    SubspacedComponent.displayName = wrapDisplayName(WrappedComponent, "Subspaced");
    return SubspacedComponent;
  };
};

/**
 * Copyright 2017, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

var ParentSpaceProvider = function ParentSpaceProvider(_ref) {
  var _ref$context = _ref.context,
      context = _ref$context === void 0 ? ReactReduxContext : _ref$context,
      children = _ref.children;
  var _context$parent = context.parent,
      ParentContext = _context$parent === void 0 ? context.Consumer ? context : ReactReduxContext : _context$parent,
      _context$child = context.child,
      ChildContext = _context$child === void 0 ? context.Consumer ? context : ReactReduxContext : _context$child;
  var parentStore = useParentSpace({
    context: ParentContext
  });
  var childContext = useReplacedContext(ChildContext, parentStore);
  return React.createElement(ChildContext.Provider, {
    value: childContext
  }, children);
};

ParentSpaceProvider.propTypes = {
  children: PropTypes.element.isRequired
};

/**
 * Copyright 2017, IOOF Holdings Limited.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

var parentSpaced = function parentSpaced(parentSpaceOptions) {
  return function (WrappedComponent) {
    var ParentSpacedComponent = function ParentSpacedComponent(props) {
      return React.createElement(ParentSpaceProvider, parentSpaceOptions, React.createElement(WrappedComponent, props));
    };

    hoistNonReactStatics(ParentSpacedComponent, WrappedComponent);
    ParentSpacedComponent.displayName = wrapDisplayName(WrappedComponent, "ParentSpaced");
    return ParentSpacedComponent;
  };
};

export { ParentSpaceProvider, SubspaceProvider, parentSpaced, subspaced, useParentSpace, useSubspace };
